package service

import (
	"context"
	"time"

	"gitea.dwysokinski.me/twhelp/dcbot/internal/domain"
)

//go:generate counterfeiter -generate

//counterfeiter:generate -o internal/mock/twhelp_service.gen.go . TWHelpService
type TWHelpService interface {
	ListVersions(ctx context.Context) ([]domain.TWVersion, error)
	ListOpenServers(ctx context.Context, version string, offset, limit int32) ([]domain.TWServer, error)
	ListClosedServers(ctx context.Context, version string, offset, limit int32) ([]domain.TWServer, error)
	GetOpenServer(ctx context.Context, versionCode, serverKey string) (domain.TWServer, error)
	GetServer(ctx context.Context, versionCode, serverKey string) (domain.TWServer, error)
	GetTribeByID(ctx context.Context, versionCode, serverKey string, id int64) (domain.Tribe, error)
	GetExistingTribeByTag(ctx context.Context, versionCode, serverKey, tribeTag string) (domain.Tribe, error)
	ListTribesByTag(ctx context.Context, versionCode, serverKey string, tribeTags []string, offset, limit int32) ([]domain.Tribe, error)
	ListVillagesByCoords(ctx context.Context, versionCode, serverKey string, coords []string, offset, limit int32) ([]domain.Village, error)
	ListEnnoblementsSince(ctx context.Context, versionCode, serverKey string, since time.Time, offset, limit int32) ([]domain.Ennoblement, error)
}
